/* 
  File: qgraphic.h
  Date: 10/15/91
  Last: 02/05/93
  Note: Header file for QGraphic Toolbox
*/

#ifndef __QGRAPHIC_DEF_
#define __QGRAPHIC_DEF_

enum graphics_errors {  /* graphresult error return codes */
  grOk             =   0,
  grNoInitGraph    =  -1,
  grNotDetected    =  -2,
  grFileNotFound   =  -3,
  grInvalidDriver  =  -4,
  grNoLoadMem      =  -5,
  grNoScanMem      =  -6,
  grNoFloodMem     =  -7,
  grFontNotFound   =  -8,
  grNoFontMem      =  -9,
  grInvalidMode    = -10,
  grError          = -11,  /* generic error */
  grIOerror        = -12,
  grInvalidFont    = -13,
  grInvalidFontNum = -14,
  grInvalidVersion = -18
};

enum graphics_drivers {  /* define graphics drivers */
  DETECT,                /* requests autodetection */
  CGA, MCGA, EGA, EGA64, EGAMONO, IBM8514,  /* 1 - 6 */
  HERCMONO, ATT400, VGA, PC3270,            /* 7 - 10 */
  CURRENT_DRIVER = -1
  };
/* ---------- Extensions ------------- */
#define VIDEO7    11  /* V7-VGA 1024i */
#define TSENG4000 12  /* TSENG ET4000 */


enum graphics_modes {  /* graphics modes for each driver */
  CGAC0      = 0,  /* 320x200 palette 0; 1 page */
  CGAC1      = 1,  /* 320x200 palette 1; 1 page */
  CGAC2      = 2,  /* 320x200 palette 2: 1 page */
  CGAC3      = 3,  /* 320x200 palette 3; 1 page */
  CGAHI      = 4,  /* 640x200 1 page   */
  MCGAC0     = 0,  /* 320x200 palette 0; 1 page */
  MCGAC1     = 1,  /* 320x200 palette 1; 1 page */
  MCGAC2     = 2,  /* 320x200 palette 2; 1 page */
  MCGAC3     = 3,  /* 320x200 palette 3; 1 page */
  MCGAMED    = 4,  /* 640x200 1 page   */
  MCGAHI     = 5,  /* 640x480 1 page   */
  EGALO      = 0,  /* 640x200 16 color 4 pages */
  EGAHI      = 1,  /* 640x350 16 color 2 pages */
  EGA64LO    = 0,  /* 640x200 16 color 1 page  */
  EGA64HI    = 1,  /* 640x350 4 color  1 page  */
  EGAMONOHI  = 0,  /* 640x350 64K on card, 1 page - 256K on card, 4 pages */
  HERCMONOHI = 0,  /* 720x348 2 pages   */
  ATT400C0   = 0,  /* 320x200 palette 0; 1 page */
  ATT400C1   = 1,  /* 320x200 palette 1; 1 page */
  ATT400C2   = 2,  /* 320x200 palette 2; 1 page */
  ATT400C3   = 3,  /* 320x200 palette 3; 1 page */
  ATT400MED  = 4,  /* 640x200 1 page   */
  ATT400HI   = 5,  /* 640x400 1 page   */
  VGALO      = 0,  /* 640x200 16 color 4 pages */
  VGAMED     = 1,  /* 640x350 16 color 2 pages */
  VGAHI      = 2,  /* 640x480 16 color 1 page  */
  PC3270HI   = 0,  /* 720x350 1 page   */
  IBM8514LO  = 0,  /* 640x480 256 colors  */
  IBM8514HI  = 1   /*1024x768 256 colors  */
  };
/* ---------- Extensions ------------- */
/* egavga.c: */
#define VGA720  3  /* 720x480 16 color */
#define VGA800  4  /* 800x600 16 color */

/* egavgahi.c: */
#define VGA1024 5  /* 1024x768 16 color (512 KB) */

/* vga256.c: */
#define VGA256LO 6 /* 320x200 256 color */
#define VGASCQ   7 /* 360x480 256 color */
#define VGA256MD 8 /* 640x400 256 color */

/* vga256hi.c: */
#define VGA256HI 9  /* 1024x768 256 color (1 MB) */

/* Colors for setpalette and setallpalette */
#if !defined(__COLORS)
#define __COLORS

enum COLORS {
  BLACK,  /* dark colors */
  BLUE,
  GREEN,
  CYAN,
  RED,
  MAGENTA,
  BROWN,
  LIGHTGRAY,
  DARKGRAY,  /* light colors */
  LIGHTBLUE,
  LIGHTGREEN,
  LIGHTCYAN,
  LIGHTRED,
  LIGHTMAGENTA,
  YELLOW,
  WHITE
  };
#endif

enum CGA_COLORS {
  CGA_LIGHTGREEN   = 1,  /* Palette C0 Color Names */
  CGA_LIGHTRED     = 2,
  CGA_YELLOW       = 3,

  CGA_LIGHTCYAN    = 1,  /* Palette C1 Color Names */
  CGA_LIGHTMAGENTA = 2,
  CGA_WHITE        = 3,

  CGA_GREEN        = 1,  /* Palette C2 Color Names */
  CGA_RED          = 2,
  CGA_BROWN        = 3,

  CGA_CYAN         = 1,  /* Palette C3 Color Names */
  CGA_MAGENTA      = 2,
  CGA_LIGHTGRAY    = 3
  };

enum EGA_COLORS {
  EGA_BLACK        =  0,  /* dark colors */
  EGA_BLUE         =  1,
  EGA_GREEN        =  2,
  EGA_CYAN         =  3,
  EGA_RED          =  4,
  EGA_MAGENTA      =  5,
  EGA_BROWN        = 20,
  EGA_LIGHTGRAY    =  7,
  EGA_DARKGRAY     = 56,  /* light colors */
  EGA_LIGHTBLUE    = 57,
  EGA_LIGHTGREEN   = 58,
  EGA_LIGHTCYAN    = 59,
  EGA_LIGHTRED     = 60,
  EGA_LIGHTMAGENTA = 61,
  EGA_YELLOW       = 62,
  EGA_WHITE        = 63
  };

enum line_styles {  /* Line styles for get/setlinestyle */
  SOLID_LINE   = 0,
  DOTTED_LINE  = 1,
  CENTER_LINE  = 2,
  DASHED_LINE  = 3,
  USERBIT_LINE = 4  /* User defined line style */
  };

enum line_widths {  /* Line widths for get/setlinestyle */
  NORM_WIDTH  = 1,
  MED_WIDTH   = 2,  /* Extension */
  THICK_WIDTH = 3
  };

enum font_names {
  DEFAULT_FONT    = 0, /* bit mapped font */
  TRIPLEX_FONT    = 1, /* "Stroked" fonts */
  SMALL_FONT      = 2,
  SANS_SERIF_FONT = 3,
  GOTHIC_FONT     = 4,
  SCRIPT_FONT     = 5,
  SIMPLEX_FONT    = 6,
  TRIP_SCR_FONT   = 7,
  COMPLEX_FONT    = 8,
  EURO_FONT       = 9,
  BOLD_FONT       = 10
};

/* Constants for EGA/VGA-ROM-Charactertables */
#define ROM8X8  -1  /* 8x8-ROM-Charactertables*/
#define ROM8X14 -2  /* 8x14-ROM-Charactertables*/
#define ROM8X16 -3  /* 8x16-ROM-Charactertables(only VGA) */

#define HORIZ_DIR 0  /* left to right */
#define VERT_DIR  1  /* bottom to top */

#define USER_CHAR_SIZE 0  /* user-defined char size */

enum fill_patterns {  /* Fill patterns for get/setfillstyle */
  EMPTY_FILL,      /* fills area in background color */
  SOLID_FILL,      /* fills area in solid fill color */
  LINE_FILL,       /* --- fill */
  LTSLASH_FILL,    /* /// fill */
  SLASH_FILL,      /* /// fill with thick lines */
  BKSLASH_FILL,    /* \\\ fill with thick lines */
  LTBKSLASH_FILL,  /* \\\ fill */
  HATCH_FILL,      /* light hatch fill */
  XHATCH_FILL,     /* heavy cross hatch fill */
  INTERLEAVE_FILL, /* interleaving line fill */
  WIDE_DOT_FILL,   /* Widely spaced dot fill */
  CLOSE_DOT_FILL,  /* Closely spaced dot fill */
  USER_FILL        /* user defined fill */
};

enum putimage_ops {  /* BitBlt operators for putimage */
  COPY_PUT,  /* MOV */
  XOR_PUT,   /* XOR */
  OR_PUT,    /* OR  */
  AND_PUT,   /* AND */
  NOT_PUT    /* NOT */
};

enum text_just {  /* Horizontal and vertical justification for settextjustify */
  LEFT_TEXT   = 0,
  CENTER_TEXT = 1,
  RIGHT_TEXT  = 2,

  BOTTOM_TEXT = 0,
/*CENTER_TEXT = 1,  already defined above */
  TOP_TEXT    = 2
};

#define MAXCOLORS 15

/* new attributes for vector-fonts */
#define QATTR_NORMAL    0
#define QATTR_BOLD      1
#define QATTR_UNDERLINE 2
#define QATTR_CURSIVE   4

struct palettetype {
  unsigned char size;
  signed char colors[MAXCOLORS+1];
  };

struct linesettingstype {
  int linestyle;
  unsigned upattern;
  int thickness;
  };

struct textsettingstype {
  int font,
  direction,
  charsize,
  horiz,
  vert;
  };

struct fillsettingstype {
  int pattern;
  int color;
  };

struct pointtype {
  int x, y;
  };

struct viewporttype {
  int left, top, right, bottom;
  int clip;
  };

struct arccoordstype {
  int x, y;
  int xstart, ystart, xend, yend;
  };

struct pcxheadertype {
  unsigned char manufacturer;
  unsigned char version;
  unsigned char encoding;
  unsigned char bits_per_pixel;
  int left, top, right, bottom;
  int hres, vres;
  unsigned char colormap[16][3];
  unsigned char reserved;
  unsigned char nplanes;
  int bytes_per_line;
  int palette_type;
  unsigned char filler[58];
  }; /* = 128 bytes */


void     far arc(int x, int y, int stangle, int endangle, int radius);
void     far bar(int left, int top, int right, int bottom);
void     far bar3d(int left, int top, int right, int bottom, int depth,
                   int topflag);
void     far circle(int x, int y, int radius);
void     far cleardevice(void);
void     far clearviewport(void);
void     far closegraph(void);
void     far detectgraph(int far *graphdriver, int far *graphmode);
void     far drawpoly(int numpoints, int far *polypoints);
void     far ellipse(int x, int y, int stangle, int endangle, int xradius,
                     int yradius);
void     far fillellipse(int x, int y, int xradius, int yradius);
void     far fillpoly(int numpoints, int far *polypoints);
void     far floodfill(int x, int y, unsigned border);
void     far getarccoords(struct arccoordstype far *arccoords);
void     far getaspectratio(int far *xasp, int far *yasp);
int      far getbkcolor(void);
int      far getcolor(void);
struct   palettetype far * far getdefaultpalette(void);
char *   far getdrivername(void);
void     far getfillpattern(char far *pattern);
void     far getfillsettings(struct fillsettingstype far *fillinfo);
int      far getgraphmode(void);
void     far getimage(int left, int top, int right, int bottom,
                      void far *bitmap);
void     far getlinesettings(struct linesettingstype far *lineinfo);
int      far getmaxcolor(void);
int      far getmaxmode(void);
int      far getmaxx(void);
int      far getmaxy(void);
char *   far getmodename(int mode_number);
void     far getmoderange(int graphdriver, int far *lomode, int far *himode);
void     far getpalette(struct palettetype far *palette);
int      far getpalettesize(void);
unsigned far getpixel(int x, int y);
unsigned far gettextattribute(void);
void     far gettextsettings(struct textsettingstype far *textinfo);
void     far getviewsettings(struct viewporttype far *viewport);
int      far getx(void);
int      far gety(void);
void     far graphdefaults(void);
char *   far grapherrormsg(int errorcode);
#if 0
void     far _graphfreemem(void far *ptr, unsigned size);
void     far * far _graphgetmem(unsigned size);
int      far installuserdriver( char far *name, int huge (*detect)(void) );
#endif
int      far graphresult(void);
unsigned far imagesize(int left, int top, int right, int bottom);
void     far initgraph(int far *graphdriver, int far *graphmode,
                       char far *pathtodriver);
int      far installuserfont(char far *name);
void     far line(int x1, int y1, int x2, int y2);
void     far linerel(int dx, int dy);
void     far lineto(int x, int y);
void     far moverel(int dx, int dy);
void     far moveto(int x, int y);
void     far outtext(char far *textstring);
void     far outtextxy(int x, int y, char far *textstring);
void     far pieslice(int x, int y, int stangle, int endangle, int radius);
void     far putimage(int left, int top, void far *bitmap, int op);
void     far putpixel(int x, int y, int color);
void     far rectangle(int left, int top, int right, int bottom);
void     far restorecrtmode(void);
void     far roundrectangle(int left, int top, int right, int bottom,
                            int radius, int fill);
void     far sector(int x, int y, int stangle, int endangle, int xradius,
                    int yradius);
void     far setactivepage(int page);
void     far setallpalette(struct palettetype far *palette);
void     far setaspectratio(int xasp, int yasp);
void     far setbkcolor(int color);
void     far setcolor(int color);
void     far setfillpattern(char far *upattern, int color);
void     far setfillstyle(int pattern, int color);
unsigned far setgraphbufsize(unsigned bufsize);
void     far setgraphmode(int mode);
void     far setlinestyle(int linestyle, unsigned pattern, int thickness);
void     far setpalette(int colornum, int color);
void     far setrgbpalette(int colornum, int red, int green, int blue);
void     far settextattribute(unsigned attribute);
void     far settextjustify(int horiz, int vert);
void     far settextstyle(int font, int direction, int charsize);
void     far setusercharsize(int multx, int divx, int multy, int divy);
void     far setviewport(int left, int top, int right, int bottom, int clip);
void     far setvisualpage(int page);
void     far setwritemode(int mode);
int      far textheight(char far *textstring);
int      far textwidth(char far *textstring);
int      far savescreen(char far *filename);
int      far loadscreen(char far *filename);
int      far moveleft (int left, int top, int right, int bottom, int count);
int      far moveright(int left, int top, int right, int bottom, int count);
int      far moveup   (int left, int top, int right, int bottom, int count);
int      far movedown (int left, int top, int right, int bottom, int count);
int      far pcxload(char far *pcxname, struct pcxheadertype far *pcxheader);
int      far pcxsave(char far *pcxname, struct pcxheadertype far *pcxheader);

/***** graphics drivers *****/
#if 0
int registerbgidriver(void (*driver)(void));
int far registerfarbgidriver(void far *__driver);
#endif

/***** graphics fonts *****/
#if 0
int registerbgifont(void (*font)(void));
int far registerfarbgifont(void far *font);
#endif

int far registerbgifont(void far *font);
int far registerfarbgifont(void far *font);

/* !!   These "functions" are NOT user-callable,   !! */
/* !! they are there just so you can link in fonts !! */

#ifdef CII
#define cdecl
#endif

void cdecl triplex_font(void);
void cdecl small_font(void);
void cdecl sansserif_font(void);
void cdecl gothic_font(void);
void cdecl script_font(void);
void cdecl simplex_font(void);
void cdecl triplexscript_font(void);
void cdecl complex_font(void);
void cdecl european_font(void);
void cdecl bold_font(void);

extern int far cdecl triplex_font_far[];
extern int far cdecl small_font_far[];
extern int far cdecl sansserif_font_far[];
extern int far cdecl gothic_font_far[];
extern int far cdecl script_font_far[];
extern int far cdecl simplex_font_far[];
extern int far cdecl triplexscript_font_far[];
extern int far cdecl complex_font_far[];
extern int far cdecl european_font_far[];
extern int far cdecl bold_font_far[];

/* #ifndef __QGRAPHIC_DEF_ */
#endif

